package com.sheepapps.sheep_auth

import android.app.Activity
import android.content.Context
import android.content.Intent
import androidx.annotation.NonNull
import androidx.core.app.ActivityCompat.startActivityForResult
import com.google.android.gms.auth.api.signin.GoogleSignIn
import com.google.android.gms.auth.api.signin.GoogleSignInClient
import com.google.android.gms.auth.api.signin.GoogleSignInOptions
import com.google.android.gms.common.api.ApiException
import io.flutter.embedding.engine.plugins.FlutterPlugin
import io.flutter.embedding.engine.plugins.activity.ActivityAware
import io.flutter.plugin.common.MethodCall
import io.flutter.plugin.common.MethodChannel
import io.flutter.plugin.common.MethodChannel.MethodCallHandler
import io.flutter.plugin.common.MethodChannel.Result
import io.flutter.plugin.common.PluginRegistry
import io.flutter.plugin.common.PluginRegistry.Registrar


/** SheepAuthPlugin */
public class SheepAuthPlugin(private val context: Context): FlutterPlugin, MethodCallHandler,
        PluginRegistry.ActivityResultListener {

  companion object {
    @JvmStatic
    fun registerWith(registrar: Registrar) {
      val channel = MethodChannel(registrar.messenger(), "sheep_auth")
      val plugin = SheepAuthPlugin(registrar.context())
      channel.setMethodCallHandler(plugin)
      activity = registrar.activity()
      registrar.addActivityResultListener(plugin)
    }
    private var SIGN_IN_GOOGLE = 0
    private var activity: Activity? = null
  }

  private lateinit var signInClientGoogle: GoogleSignInClient
  private lateinit var result: Result
  private var googleIdToken: String? = null

  override fun onAttachedToEngine(@NonNull flutterPluginBinding: FlutterPlugin.FlutterPluginBinding) {
    val channel = MethodChannel(flutterPluginBinding.flutterEngine.dartExecutor, "sheep_auth")
    channel.setMethodCallHandler(this)
  }

  // This static function is optional and equivalent to onAttachedToEngine. It supports the old
  // pre-Flutter-1.12 Android projects. You are encouraged to continue supporting
  // plugin registration via this function while apps migrate to use the new Android APIs
  // post-flutter-1.12 via https://flutter.dev/go/android-project-migration.
  //
  // It is encouraged to share logic between onAttachedToEngine and registerWith to keep
  // them functionally equivalent. Only one of onAttachedToEngine or registerWith will be called
  // depending on the user's project. onAttachedToEngine or registerWith must both be defined
  // in the same class.

  override fun onMethodCall(@NonNull call: MethodCall, @NonNull result: Result) {
    this.result = result
    when(call.method.toString()) {
      "configure" -> configure(call)
      "auth_with_google" -> authWithGoogle()
      else -> result.notImplemented()
    }
  }

  private fun configure(call: MethodCall) {
    googleIdToken = call.argument("google_id_token")
    if (googleIdToken != null) {
      val signInOptions = GoogleSignInOptions.Builder(GoogleSignInOptions.DEFAULT_SIGN_IN)
              .requestIdToken(googleIdToken)
              .requestEmail()
              .build()
      signInClientGoogle = GoogleSignIn.getClient(context, signInOptions)
      result.success(null)
    }
  }

  private fun authWithGoogle() {
    if (googleIdToken != null) {
      val lastSignedIn = GoogleSignIn.getLastSignedInAccount(context)
      if (lastSignedIn != null && GoogleSignIn.hasPermissions(lastSignedIn)) {
        val user = HashMap<String, String?>()
        user["name"] = lastSignedIn.displayName
        user["email"] = lastSignedIn.email
        user["id"] = lastSignedIn.id
        user["token"] = lastSignedIn.idToken
        user["photo_url"] = lastSignedIn.photoUrl.toString()
        result.success(user)
      } else {
        val signInIntent = signInClientGoogle.signInIntent
        activity!!.startActivityForResult(signInIntent, SIGN_IN_GOOGLE)
      }
    } else {
      result.error("google_auth", "Google token is not provided", "error")
    }
  }


  override fun onActivityResult(requestCode: Int, resultCode: Int, data: Intent): Boolean {
    when (requestCode) {
      SIGN_IN_GOOGLE -> {
        try {
          val task = GoogleSignIn.getSignedInAccountFromIntent(data)
          val signedInAccount = task.getResult(ApiException::class.java)!!
          val user = HashMap<String, String?>()
          user["name"] = signedInAccount.displayName
          user["email"] = signedInAccount.email
          user["id"] = signedInAccount.id
          user["token"] = signedInAccount.idToken
          user["photo_url"] = signedInAccount.photoUrl.toString()
          result.success(user)
        } catch (e: ApiException) {
          result.error("google_auth", e.toString(), "error")
        }
      }
    }
    return true
  }

  override fun onDetachedFromEngine(@NonNull binding: FlutterPlugin.FlutterPluginBinding) {
  }

}
